/*  SpiralSynth
 *  Copyleft (C) 2000 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Check_Button.H>
#include "Widgets/Fl_Knob.H"

#include "GUIBase.h"
#include "../SpiralSound/Sequencer.h"

#ifndef SEQGUI
#define SEQGUI

class SequencerGUI : public GUIBase
{
public:
	SequencerGUI(Sequencer *o);
	
	virtual void CreateGUI(int xoff, int yoff, char *name);
	
	Sequencer *m_seq;	
	
private:

	Fl_Group 		*GUISeqGroup;	
	Fl_Button 		*Event[16][12];
	int 			 EventTable[16][12];
	bool 			 EventList[191];
	
	Fl_Knob 		*Speed;
	Fl_Knob 		*Octave;
	Fl_Knob 		*Length;
	Fl_Knob 		*ArpNotes;
	Fl_Button 		*Arp;
	Fl_Button 		*Play;
	Fl_Button 		*Stop;
	Fl_Button 		*ArpType;
		
	//// Callbacks ////
	
	inline void cb_Event_i(Fl_Button* o, void *v);
	static void cb_Event  (Fl_Button* o, void *v);
	inline void cb_Play_i(Fl_Button* o, void* v); 
	static void cb_Play(Fl_Button* o, void* v); 
	inline void cb_Speed_i(Fl_Knob* o, void* v); 
	static void cb_Speed(Fl_Knob* o, void* v);
	inline void cb_Octave_i(Fl_Knob* o, void* v);
	static void cb_Octave(Fl_Knob* o, void* v); 
	inline void cb_Length_i(Fl_Knob* o, void* v); 
	static void cb_Length(Fl_Knob* o, void* v); 
	inline void cb_Arp_i(Fl_Button* o, void* v); 
	static void cb_Arp(Fl_Button* o, void* v); 
	inline void cb_ArpType_i(Fl_Button* o, void* v); 
	static void cb_ArpType(Fl_Button* o, void* v); 
	inline void cb_ArpNotes_i(Fl_Knob* o, void* v); 
	static void cb_ArpNotes(Fl_Knob* o, void* v); 
};

#endif
