// generated by Fast Light User Interface Designer (fluid) version 2.00

#ifndef Fl_Knob_H
#define Fl_Knob_H
#include <FL/Fl.H>
#include <FL/Fl_Valuator.H>

class Fl_Knob : public Fl_Valuator {
public:
  enum Fl_Knobtype {DOTLIN=0,DOTLOG_1,DOTLOG_2,DOTLOG_3,LINELIN,LINELOG_1,LINELOG_2,LINELOG_3};
private:
  int _type;
  float _percent;
  int _scaleticks;
  short a1,a2;
public:
  Fl_Knob(int xx,int yy,int ww,int hh,const char *l=0);
  ~Fl_Knob();
private:
  void draw();
  int handle(int  event);
public:
  void type(int ty);
private:
  void shadow(const int offs,const uchar r,uchar g,uchar b);
  void draw_scale(const int ox,const int oy,const int side);
  void draw_cursor(const int ox,const int oy,const int side);
public:
  void cursor(const int pc);
  void scaleticks(const int tck);
};
#endif
