g++ -shared -o SpiralSynth.so Synth.o SpiralInfo.o SpiralSound/ModBase.o SpiralSound/iir_filter.o SpiralSound/Filter.o SpiralSound/Amp.o SpiralSound/Mixer.o SpiralSound/Delay.o SpiralSound/Envelope.o SpiralSound/Oscillator.o SpiralSound/Output.o SpiralSound/NoteTable.o SpiralSound/Sequencer.o SpiralSound/Midi.o SpiralSound/RiffWav.o GUI/GUIBase.o GUI/OscillatorGUI.o GUI/EnvelopeGUI.o GUI/FilterGUI.o GUI/MixerGUI.o GUI/DelayGUI.o GUI/SequencerGUI.o GUI/OutputGUI.o GUI/ScopeGUI.o GUI/PatchBankGUI.o GUI/RouteGUI.o GUI/Widgets/Fl_Knob.o  -L/usr/X11R6/lib -lfltk -lGL -lXext -lX11 -lm
